/*------------------------------------------------------------------------
 Module:        C:\lcc\srcs\convex\CONVEX.C
 Author:        rymaeda@yahoo.com - http://www.geocities.com/rymaeda
 Project:       Sem Nome
 State:         Implementacao inicial
 Creation Date: 10/07/2000
 Description:   Programa para teste de pontos,
                verificando se sao interiores ou nao a um
                triangulo utilizando a teoria da
                "Combinacao Convexa" (Algebra Linear).
                Apesar do nome medonho e da implementacao
                confusa a teoria eh relativamente
                simples. Mas *nao* tente entender a
                teoria por esta implementacao.
		Revisoes-
		20/09/2000: Expressoes fatoradas.
------------------------------------------------------------------------*/
#include <stdio.h>
#include <time.h>

typedef int XYCOORD;

XYCOORD xa, ya, xb, yb, xc, yc, xmax, ymax, xmin, ymin, beta;

/*------------------------------------------------------------------------
 Procedure:     DefineTriangulo ID:1
 Purpose:       Funcao que armazena as coordenadas dos vertices do
                triangulo e calcula o valor de beta. Notar que os
                vertices dos triangulos podem vir em qualquer ordem.
 Input:         Coordenadas dos vertices do triangulo
                (xa, ya) (xb, yb) (xc, yc).
 Output:        Alteracao de todas as variaveis globais.
 Errors:        Podem ocorrer erros por 'overflow'.
------------------------------------------------------------------------*/
void DefineTriangulo(XYCOORD x1, XYCOORD y1, XYCOORD x2, XYCOORD y2, XYCOORD x3, XYCOORD y3)
{
	xmax= xmin= xa = x1;
	ymax= ymin= ya = y1;
	xb = x2;if (xb>xmax) xmax=xb; else if (xb<xmin)	xmin= xb;
	yb = y2;if (yb>ymax) ymax=yb; else if (yb<ymin) ymin= yb;
	xc = x3;if (xc>xmax) xmax=xc; else if (xc<xmin) xmin= xc;
	yc = y3;if (yc>xmax) ymax=yc; else if (yc<xmin) ymin= yc;
	beta = ya*(xc-xb)+yb*(xa-xc)+yc*(xb-xa);
	printf("\nTriangulo= (%3d, %3d) (%3d, %3d) (%3d, %3d)\n",\
		 xa, ya, xb, yb, xc, yc);
}

/*------------------------------------------------------------------------
 Procedure:     TestaPonto ID:1
 Purpose:       Verifica se o ponto pertence ao interior do triangulo
                Notar que de beta so se usa o sinal.
                Os pontos sobre as arestas serao considerados
                internos, para considera-los nao internos basta
                substituir os '<' por '<=' e os '>' por '>='.
                Observe que, efetivamente, esta funcao verifica se os
                alphas (valores testados nos 'if's) teem o mesmo
                sinal de beta.
                Tentou-se otimizar a velocidade do codigo.
 Input:         Coordenadas (xe, ye) do ponto a testar.
 Output:        Retorna 1 se o ponto pertencer ao triangulo e 0
                (zero) caso contrario.
 Errors:        Podem ocorrer erros por 'overflow'.
------------------------------------------------------------------------*/
int TestaPonto(XYCOORD xe, XYCOORD ye)
{
	if ((xe>xmax)||(xe<xmin)||(ye>ymax)||(ye<ymin))
		return 0;
	if (beta > 0) {
		if ((yb*(xe-xc)+yc*(xb-xe)+ye*(xc-xb)) < 0)
			return 0;
		if ((ya*(xc-xe)+yc*(xe-xa)+ye*(xa-xc)) < 0)
			return 0;
		if ((ya*(xe-xb)+yb*(xa-xe)+ye*(xb-xa)) < 0)
			return 0;
	}
	else {
		if ((yb*(xe-xc)+yc*(xb-xe)+ye*(xc-xb)) > 0)
			return 0;
		if ((ya*(xc-xe)+yc*(xe-xa)+ye*(xa-xc)) > 0)
			return 0;
		if ((ya*(xe-xb)+yb*(xa-xe)+ye*(xb-xa)) > 0)
			return 0;
	}
	return 1;
}

void MostraEstado(int xe, int ye)
{
	printf("O ponto (%3d, %3d) esta ", xe, ye);
	if (TestaPonto(xe, ye))
		printf("(Dentro).\n");
	else
	    printf("( ) Fora.\n");
}

int main(void)
{

	DefineTriangulo(0, 0, 10, 10, 10, 0);
	/*
	           (10,10)
        	/|
	       / |
	      /  |
	(0,0)----- (10,0)
	*/
	MostraEstado(5, 5);
	MostraEstado(20, 5);
	MostraEstado(2, 1);
	MostraEstado(-1, 1);
	MostraEstado(5, 2);
	MostraEstado(5, 60);

	DefineTriangulo(0, 0, 0, 10, 10, 0);
	/*
	       (0,10)
	     |\
	     | \
	     |  \
	(0,0)----- (10,0)
	*/
	MostraEstado(5, 5);
	MostraEstado(20, 5);
	MostraEstado(2, 1);
	MostraEstado(-1, 1);
	MostraEstado(5, 2);
	MostraEstado(5, 60);
	return 0;
}
