/*------------------------------------------------------------------------
 Module:        C:\lcc\srcs\convex\CONVEX3D.C
 Author:        rymaeda@yahoo.com - http://www.geocities.com/rymaeda
 Project:       Sem Nome
 State:         Implementacao inicial
 Creation Date: 14/07/2000
 Description:   Programa para teste de pontos,
                verificando se sao interiores ou nao a um
                tetraedro utilizando a teoria da
                "Combinacao Convexa" (Algebra Linear).
                Apesar do nome medonho e da implementacao
                confusa a teoria eh relativamente
                simples. Mas *nao* tente entender a
                teoria por esta implementacao.
		Revisoes-
		20/09/2000: Expressoes fatoradas.
------------------------------------------------------------------------*/
#include <stdio.h>

typedef int COORD;

COORD xa, ya, za,
      xb, yb, zb,
	  xc, yc, zc,
	  xd, yd, zd,
	  beta;

/*------------------------------------------------------------------------
 Procedure:     DefineTriangulo ID:1
 Purpose:       Funcao que armazena as coordenadas dos vertices do
                tetraedro e calcula o valor de beta. Notar que os
                vertices podem vir em qualquer ordem.
 Input:         Coordenadas dos vertices do triangulo
                (xa, ya, za) (xb, yb, zb) (xc, yc, zc) (xd , yd, zd).
 Output:        Alteracao de todas as variaveis globais.
 Errors:        Podem ocorrer erros por 'overflow'.
------------------------------------------------------------------------*/
void DefineTetraedro(COORD x1, COORD y1, COORD z1, COORD x2, COORD y2, \
	COORD z2, COORD x3, COORD y3, COORD z3, COORD x4, COORD y4, COORD z4)
{
	xa = x1;ya = y1;za = z1;
	xb = x2;yb = y2;zb = z2;
	xc = x3;yc = y3;zc = z3;
	xd = x4;yd = y4;zd = z4;
	beta = (ya*(xb*(zd-zc)+xc*(zb-zd)+xd*(zc-zb))+yb*(xa*(zc-zd)+\
		xc*(zd-za)+xd*(za-zc))+yc*(xa*(zd-zb)+xb*(za-zd)+\
		xd*(zb-za))+yd*(xa*(zb-zc)+xb*(zc-za)+xc*(za-zb)));
	/*printf("Beta %d\n",beta);*/
	printf("\nTetraedro= (%3d, %3d, %3d) (%3d, %3d, %3d) "
		"(%3d, %3d, %3d) (%3d, %3d, %3d)\n", xa, ya, za,\
		xb, yb, zb, xc, yc, zc, xd, yd, zd);
}

/*------------------------------------------------------------------------
 Procedure:     TestaPonto ID:1
 Purpose:       Verifica se o ponto pertence ao interior do tetraedro
                Notar que de beta so se usa o sinal.
                Os pontos sobre as faces serao considerados
                internos, para considera-los nao internos basta
                substituir os '<' por '<=' e os '>' por '>='.
                Observe que, efetivamente, esta funcao verifica se os
                alphas (valores testados nos 'if's) teem o mesmo
                sinal de beta.
                Tentou-se otimizar a velocidade do codigo.
 Input:         Coordenadas (xe, ye, ze) do ponto a testar.
 Output:        Retorna 1 se o ponto pertencer ao tetraedro e 0
                (zero) caso contrario.
 Errors:        Podem ocorrer erros por 'overflow'.
------------------------------------------------------------------------*/
int TestaPonto(COORD xe, COORD ye, COORD ze)
{
	if (beta > 0) {
		if ((((ze-zb)*xc+(zc-ze)*xb+(zb-zc)*xe)*yd+(xb*(zd-zc)+\
			xc*(zb-zd)+xd*(zc-zb))*ye+((zc-zd)*xe+(ze-zc)*xd+\
			(zd-ze)*xc)*yb+((zd-zb)*xe+(zb-ze)*xd+(ze-zd)*xb)*yc) < 0)
			return 0;
		if ((((zd-zc)*xe+(ze-zd)*xc+(zc-ze)*xd)*ya+((za-ze)*xc+\
			(ze-zc)*xa+(zc-za)*xe)*yd+(xa*(zc-zd)+xc*(zd-za)+\
			xd*(za-zc))*ye+((zd-ze)*xa+(ze-za)*xd+(za-zd)*xe)*yc) < 0)
			return 0;
		if ((((zb-zd)*xe+(zd-ze)*xb+(ze-zb)*xd)*ya+((ze-za)*xb+\
			(zb-ze)*xa+(za-zb)*xe)*yd+(xa*(zd-zb)+xb*(za-zd)+\
			xd*(zb-za))*ye+((ze-zd)*xa+(za-ze)*xd+(zd-za)*xe)*yb) < 0)
			return 0;
		if ((((zb-ze)*xc+(ze-zc)*xb+(zc-zb)*xe)*ya+((zb-zc)*xa+\
			(za-zb)*xc+(zc-za)*xb)*ye+((za-zc)*xe+(zc-ze)*xa+\
			(ze-za)*xc)*yb+((zb-za)*xe+(ze-zb)*xa+(za-ze)*xb)*yc) < 0)
			return 0;
	}
	else {
		if ((((ze-zb)*xc+(zc-ze)*xb+(zb-zc)*xe)*yd+(xb*(zd-zc)+\
			xc*(zb-zd)+xd*(zc-zb))*ye+((zc-zd)*xe+(ze-zc)*xd+\
			(zd-ze)*xc)*yb+((zd-zb)*xe+(zb-ze)*xd+(ze-zd)*xb)*yc) > 0)
			return 0;
		if ((((zd-zc)*xe+(ze-zd)*xc+(zc-ze)*xd)*ya+((za-ze)*xc+\
			(ze-zc)*xa+(zc-za)*xe)*yd+(xa*(zc-zd)+xc*(zd-za)+\
			xd*(za-zc))*ye+((zd-ze)*xa+(ze-za)*xd+(za-zd)*xe)*yc) > 0)
			return 0;
		if ((((zb-zd)*xe+(zd-ze)*xb+(ze-zb)*xd)*ya+((ze-za)*xb+\
			(zb-ze)*xa+(za-zb)*xe)*yd+(xa*(zd-zb)+xb*(za-zd)+\
			xd*(zb-za))*ye+((ze-zd)*xa+(za-ze)*xd+(zd-za)*xe)*yb) > 0)
			return 0;
		if ((((zb-ze)*xc+(ze-zc)*xb+(zc-zb)*xe)*ya+((zb-zc)*xa+\
			(za-zb)*xc+(zc-za)*xb)*ye+((za-zc)*xe+(zc-ze)*xa+\
			(ze-za)*xc)*yb+((zb-za)*xe+(ze-zb)*xa+(za-ze)*xb)*yc) > 0)
			return 0;
	}
	return 1;
}

void MostraEstado(int xe, int ye, int ze)
{
	printf("O ponto (%3d, %3d, %3d) esta ", xe, ye, ze);
	if (TestaPonto(xe, ye, ze))
		printf("(Dentro).\n");
	else
	    printf("( ) Fora.\n");
}

int main(void)
{
	DefineTetraedro(00, 00, 00,\
   			10, 10, 00,\
			10, 00, 00,\
			00, 00, 10);
	MostraEstado(4, 5, 1);
	MostraEstado(5, 5, 1);
	MostraEstado(6, 5, 1);
	MostraEstado(20, 5, 1);
	MostraEstado(2, 1, 1);
	MostraEstado(-1, 1, 1);
	MostraEstado(5, 2, 1);
	MostraEstado(5, 60, 1);

	DefineTetraedro(-10, -10, -10,\
   			10, 10, 00,\
			10, 00, -10,\
			00, 00, 10);
	MostraEstado(4, 5, 1);
	MostraEstado(5, 5, 1);
	MostraEstado(6, 5, 1);
	MostraEstado(20, 5, 1);
	MostraEstado(2, 1, 1);
	MostraEstado(-1, 1, 1);
	MostraEstado(5, 2, 1);
	MostraEstado(5, 60, 1);
	return 0;
}
