/*------------------------------------------------------------------------
 Module:        C:\lcc\srcs\convex\INTERSEC.C
 Author:        rymaeda@yahoo.com - http://www.geocities.com/rymaeda
 Project:       Sem Nome
 State:         Implementacao inicial
		Corrigido "intercepcao" (Arrrghh!!) 31/03/2002
 Creation Date: 15/07/2000
 Description:   Programa para teste de interseccao de
                segmentos,verificando se dois segmentos
                de reta se interseccionam ou nao.
------------------------------------------------------------------------*/
#include <stdio.h>

typedef int COORD;

static COORD A1, A2, A3, A4, gx1, gx3, gy1, gy3;

float ParamSeg1, ParamSeg2;

/*------------------------------------------------------------------------
 Procedure:     DefineSegmento1 ID:1
 Purpose:       Funcao que armazena as coordenadas do primeiro
                segmento. Os extremos podem vir em qualquer ordem.
 Input:         Coordenadas dos extremos (x1, y1) (x2, y2).
 Output:        Alteracao das variaveis globais A1, A3, gx1 e gy1.
 Errors:        Desconhecidos.
------------------------------------------------------------------------*/
void DefineSegmento1(COORD x1, COORD y1, COORD x2, COORD y2)
{
	A1= x2-x1;
	A3= y2-y1;
	gx1= x1;
	gy1= y1;
}

/*------------------------------------------------------------------------
 Procedure:     DefineSegmento2 ID:1
 Purpose:       Funcao que armazena as coordenadas do segundo
                segmento. Os extremos podem vir em qualquer ordem.
 Input:         Coordenadas dos extremos (x3, y3) (x4, y4).
 Output:        Alteracao das variaveis globais A2, A4, gx3 e gy3.
 Errors:        Desconhecidos.
------------------------------------------------------------------------*/
void DefineSegmento2(COORD x3, COORD y3, COORD x4, COORD y4)
{
	A2= x4-x3;
	A4= y4-y3;
	gx3= x3;
	gy3= y3;
}

/*------------------------------------------------------------------------
 Procedure:     X_intersec ID:1
 Purpose:       Calcula a coordenada 'x' do ponto de interseccao
                entre as duas *retas* que contem os segmentos
                testados.
 Input:         Nenhum
 Output:        Coordenada 'x' da interseccao.
 Errors:        Deve ser chamado apos a funcao TestaIntersec(), pois
                precisa de ParamSeg1.
------------------------------------------------------------------------*/
COORD X_intersec(void){
	return (COORD)((float)(gx1+ParamSeg1*A1));
}

/*------------------------------------------------------------------------
 Procedure:     Y_intersec ID:1
 Purpose:       Calcula a coordenada 'y' do ponto de interseccao
                entre as duas *retas* que contem os segmentos
                testados.
 Input:         Nenhum
 Output:        Coordenada 'y' da interseccao.
 Errors:        Deve ser chamado apos a funcao TestaIntersec(), pois
                precisa de ParamSeg1.
------------------------------------------------------------------------*/
COORD Y_intersec(void){
	return (COORD)((float)(gy1+ParamSeg1*A3));
}

/*------------------------------------------------------------------------
 Procedure:     TestaIntersec ID:1
 Purpose:       Verifica se os segmentos cruzam-se.
 Input:         Utiliza variaveis globais.
 Output:        Retorna 1 se os segmentos cruzarem, 0
                (zero) caso contrario e -1 se forem paralelos
				mesmo se forem sobrepostos..
 Errors:        Podem ocorrer erros por 'overflow'.
                E caso os segmentos estejam sobrepostos resulta em
                epsilon=0, e retorna -1.
------------------------------------------------------------------------*/
int TestaIntersec(void){
	float epsilon;
	COORD B1, B2;
	/* epsilon sera igual a zero sempre que os dois segmentos forem paralelos.
       O proximo 'if' retornara -1 *sempre* que gamma for igual a zero,
	   e isto pode ser **errado**, pois existe a possibilidade de os dois
	   segmentos estarem sobrepostos, que eh um caso particular de
	   paralelismo e interseccao, situacao em que existirao infinitos pontos
	   comuns, ou seja uma interseccao de infinitos pontos.
	   Mas este eh um problema que depende fortemente de como o programa
	   podera utilizar esta informacao ou mesmo se precisa desta informacao.
	*/
	if (!(epsilon= A1*A4-A2*A3))
		return -1;
	B1= gx3-gx1;
	B2= gy3-gy1;
	ParamSeg1= (float)(A4*B1-A2*B2)/epsilon;
	ParamSeg2= (float)(B1*A3-A1*B2)/epsilon;
	if ((0<=ParamSeg1)&&(ParamSeg1<=1)&&(0<=ParamSeg1)&&(ParamSeg1<=1))
		return 1;
	else
		return 0;
}

void MostraEstado(void)
{
	switch (TestaIntersec()){
		case 1:
			printf("Segmentos cruzam.\n");
			printf("Interseccao dos segmentos em");
			break;
		case -1:
			printf("Retas paralelas.\n\n");
			return;
		case 0:
			printf("Segmentos nao cruzam.\n");
			printf("Interseccao de retas em");
	}
	printf(" (%3d, %3d)\n\n",X_intersec(), Y_intersec());
}

int main(void)
{
	DefineSegmento1(0, 0, 100, 100);
	DefineSegmento2(0, 100, 100, 0);
	MostraEstado();

	DefineSegmento2(40, 100, 100, 0);
	MostraEstado();

	DefineSegmento1(0, 0, 1000, 100);
	MostraEstado();

	DefineSegmento1(0, 0, 10, 100);
	DefineSegmento2(200, 110, 300, 110);
	MostraEstado();

	DefineSegmento1(0, 0, 100, 100);
	DefineSegmento2(0, 10, 200, 200);
	MostraEstado();

	/* Caso em que o teste falha: sobreposicao de segmentos */
	DefineSegmento1(0, 0, 100, 100);
	DefineSegmento2(20, 20, 80, 80);
	MostraEstado();

	return 0;
}
